
__title__ = 'aeat-web-services'
__summary__ = 'Spanish Tax Agency Electronic Office (AEAT) Integration'
__version__ = '2.0.2'
__license__ = 'MIT'
__uri__ = 'https://github.com/initios/aeat-web-services'
__author__ = 'Initios Desarrollos'
__email__ = 'dev@initios.com'
